package org.eclipse.palamedes.kif.core.parser;

import java_cup.runtime.*;

%%

%class KIFScanner
%cup
%ignorecase
%line
%column
%full

%eofval{
  return symbol(KIFSymbols.EOF);
%eofval}

%{
  private final Symbol symbol(int type) {
    return new Symbol(type, zzStartRead, zzMarkedPos);
  }

  private final Symbol symbol(int type, Object value) {
    return new Symbol(type, zzStartRead, zzMarkedPos, value);
  }
%}

%state COMMENT

LineTerminator    = \r|\n|\r\n
InputCharacter    = [^\r\n]
WhiteSpace        = {LineTerminator}+ | [ \t\f]+

SingleComment     = ";" {InputCharacter}* {LineTerminator}
Comment           = {SingleComment}+

Variable          = "?" [\_a-zA-Z] [^\t\f\ \r\n\(\)\;]*
Constant          = [^\t\f\ \r\n\(\)\;]*
%%

/* keywords */

<YYINITIAL> {
  "("                         { return symbol( KIFSymbols.OPEN);   }
  ")"                         { return symbol( KIFSymbols.CLOSE);  }
  "not"                       { return symbol( KIFSymbols.NOT);    }
  "and"                       { return symbol( KIFSymbols.AND);    }
  "or"                        { return symbol( KIFSymbols.OR);     }
  "<="                        { return symbol( KIFSymbols.IMP);    }

  {Variable}                  { return symbol( KIFSymbols.VARIABLE,
                                               yytext() ); }

  {Constant}                  { return symbol( KIFSymbols.CONSTANT,
                                               yytext() ); }

  {Comment}                   { System.out.println("Comment[" + yytext() + "]");
                                return symbol( KIFSymbols.COMMENT,
                                               yytext()); }

  {WhiteSpace}                { /*System.out.println("WhiteSpace[" + yytext() + "]");*/
                                /*return symbol( KIFSymbols.WHITESPACE,
                                               yytext());*/ }

}

/* error fallback */
.|\n                             { throw new Error( "Illegal character <" +
                                                    yytext() +
                                                    "> in line " +
                                                    yyline +
                                                    " character " +
                                                    yycolumn ); }
