package org.eclipse.palamedes.kif.core.ast;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.palamedes.kif.core.ast.visitor.NotationWriter;

public class KIFForm extends ASTNode {

    private final KIFSeq<KIFSent>  sentences = new KIFSeq<KIFSent>();
    private final List<KIFComment> comments  = new ArrayList<KIFComment>();
    private final List<ASTNode>    children  = new ArrayList<ASTNode>();

    protected KIFForm(AST ast) {
        super(ast);
    }

    public KIFSeq<KIFSent> getSentences() {
        return sentences;
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.kif.core.ast.ASTNode#hasChildren()
     */
    public boolean hasChildren() {
        return !children.isEmpty();
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.kif.core.ast.ASTNode#getChildren()
     */
    public List<ASTNode> getChildren() {
        return children;
    }

    public boolean isEmpty() {
        return sentences.isEmpty();
    }

    @Override
    ASTNode clone0(AST ast) {
        KIFForm newNode = ast.newForm(sentences.clone0(ast));

        // So far comments are not used for refactoring
        // -> just copy the comments
        newNode.comments.addAll(comments);
        newNode.children.addAll(children);
        return newNode;
    }

    @Override
    final int getNodeType0() {
        return ASTNode.FORM;
    }

    @Override
    public void traverse(ASTVisitor visitor) {
        visitor.visit(this);
    }

    @Override
    boolean subtreeMatch0(ASTMatcher matcher, Object other) {
        return matcher.match(this, other);
    }

    @Override
    public String toString() {
        if ( children.isEmpty() )
            return "";

        return NotationWriter.toDefaultString(this);
    }

    public List<KIFComment> getComments() {
        return comments;
    }

    /**
     * This method returns a new list that contains only the top level
     * implications of the form.
     *
     * @return KIFSeq<KIFImplication> list of top level implications.
     */
    public KIFSeq<KIFImplication> getImplications() {
        KIFSeq<KIFImplication> newList = new KIFSeq<KIFImplication>();
        for (KIFSent sentence : sentences)
            if (sentence.getNodeType() == KIFSent.IMPLICATION)
                newList.add( (KIFImplication) sentence);
        return newList;
    }


    protected final void copyMembers(KIFForm node) {
        super.copyMembers(node);
        sentences.addAll(node.sentences);
        comments.addAll(node.comments);
        children.addAll(node.children);
    }
}
