package org.eclipse.palamedes.kif.core.ast;


public class KIFImplication extends KIFLogSent {

    private       KIFSent 		  consequence;
    private final KIFSeq<KIFSent> premise     = new KIFSeq<KIFSent>();

    protected KIFImplication(AST ast) {
        super(ast);
    }

    public KIFSent getConsequence() {
        return consequence;
    }

    public void setConsequence(KIFSent consequence) {
        this.consequence = consequence;
    }

    @Override
    public String getName() {
        return "<=";
    }

    public final KIFSeq<KIFSent> getPremise() {
        return premise;
    }

    @Override
    ASTNode clone0(AST ast) {
        KIFImplication newNode = ast.newImplication(
                                            (KIFSent) consequence.clone(ast),
                                                      premise.clone0(ast) );
        newNode.setSourceStart ( getSourceStart()  );
        newNode.setSourceLength( getSourceLength() );
        return newNode;
    }

    @Override
    final int getNodeType0() {
        return ASTNode.IMPLICATION;
    }

    @Override
    public void traverse(ASTVisitor visitor) {
        visitor.visit(this);
    }

    @Override
    boolean subtreeMatch0(ASTMatcher matcher, Object other) {
        return matcher.match(this, other);
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder("(<= ");
        sb.append(consequence.toString());
        if ( !premise.isEmpty() ) {
            sb.append("\n");
            for (Object obj : premise)
                sb.append("\t").append(obj.toString()).append("\n");

            // remove last line break
            sb.deleteCharAt( sb.length()-1 );
        }
        return sb.append(')').toString();
    }

    @Override
    public int getOperandCount() {
        return premise.size() + 1;
    }

    @Override
    public KIFSeq<KIFSent> getOperands() {
        KIFSeq<KIFSent> sequence = new KIFSeq<KIFSent>();
        sequence.add(consequence);
        sequence.addAll(premise);
        return sequence;
    }

    protected final void copyMembers(KIFImplication node) {
        super.copyMembers(node);
        consequence = node.consequence;
        premise.addAll(node.premise);
        premise.trimToSize();
    }
}
