package org.eclipse.palamedes.kif.core.ast.visitor;

import org.eclipse.palamedes.kif.core.ast.ASTNode;
import org.eclipse.palamedes.kif.core.ast.KIFIndVar;

/**
 * The GroundChecker recursively checks whether a given ASTNode contains a
 * variable. This class serves the purpose nevertheless its quite inefficient 
 * since it keeps checking even after a variable was found.
 * 
 * The next implementation should create an exception and stop searching after a 
 * variable was found. To be even more efficient an static exception should 
 * be thrown to avoid creating a stack report for the exceptions every time.
 *
 * @author Ingo Keller
 */
public class GroundChecker extends DefaultVisitor {

    boolean isGround;

    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.kif.core.ast.visitor.DefaultVisitor#visit(org.eclipse.palamedes.kif.core.ast.KIFIndVar)
     */
    @Override
    public void visit(KIFIndVar node) {
        isGround = false;
    }

    /**
     * This method checks the given ASTNode whether it contains a KIFIndVar or 
     * not.
     * @param node
     * @return Returns true if a KIFIndVar was found, otherwise false;
     */
    public boolean isGround(ASTNode node) {
        isGround = true;
        node.traverse(this);
        return isGround;
    }

}
